disp(' ')
disp(' arbit.m   ver 2.6   January 3, 2011')
disp(' by Tom Irvine   Email: tomirvine@aol.com')
disp(' ')
disp(' This program calculates the response of a single-degree-of-freedom')
disp(' system to an arbitrary base input time history. ')
disp(' ')
disp(' The input time history must have two columns: time(sec) & accel(G)')
disp(' ')
%
clear omega;
clear omegad;
clear dt;
clear sr;
clear damp;
clear a1;
clear a2;
clear b1;
clear b2;
clear c1;
clear c2;
clear sum;
clear t;
clear tt;
clear y;
clear yy;
clear THM;
clear x_pos;
clear x_neg;
clear rd_pos;
clear rd_neg;
clear a_resp;
clear x_resp;
clear forward;
clear back;
clear rd_forward;
clear rd_back;
clear length;
%
disp(' Select file input method ');
disp('   1=external ASCII file ');
disp('   2=file preloaded into Matlab ');
disp('   3=Excel file ');
file_choice = input('');
%
if(file_choice==1)
        [filename, pathname] = uigetfile('*.*');
        filename = fullfile(pathname, filename);
        fid = fopen(filename,'r');
        THM = fscanf(fid,'%g %g',[2 inf]);
        THM=THM';
end
if(file_choice==2)
        THM = input(' Enter the matrix name:  ');
end
if(file_choice==3)
        [filename, pathname] = uigetfile('*.*');
        xfile = fullfile(pathname, filename);
%        
        THM = xlsread(xfile);
%         
end
%
t=THM(:,1);
y=THM(:,2);
%
tmx=max(t);
tmi=min(t);
n = length(y);
dt=(tmx-tmi)/(n-1);
sr=1./dt;
%
disp(' ')
disp(' Time Step ');
dtmin=min(diff(t));
dtmax=max(diff(t));
%
out4 = sprintf(' dtmin  = %8.4g sec  ',dtmin);
out5 = sprintf(' dt     = %8.4g sec  ',dt);
out6 = sprintf(' dtmax  = %8.4g sec  ',dtmax);
disp(out4)
disp(out5)
disp(out6)
%
disp(' ')
disp(' Sample Rate ');
out4 = sprintf(' srmin  = %8.4g samples/sec  ',1/dtmax);
out5 = sprintf(' sr     = %8.4g samples/sec  ',1/dt);
out6 = sprintf(' srmax  = %8.4g samples/sec  \n',1/dtmin);
disp(out4)
disp(out5)
disp(out6)
%
ncontinue=1;
if(((dtmax-dtmin)/dt)>0.01)
    disp(' ')
    disp(' Warning:  time step is not constant.  Continue calculation? 1=yes 2=no ')
    ncontinue=input(' ')
end
if(ncontinue==1)
%
disp(' ')
fn(1)=input(' Enter the natural frequency (Hz)  ');
%
disp(' ')
idamp=input(' Enter damping format:  1= damping ratio   2= Q  ');	
%
disp(' ')
if(idamp==1)
    damp=input(' Enter damping ratio (typically 0.05) ');
else
    Q=input(' Enter the amplification factor (typically Q=10) ');
    damp=1./(2.*Q);
end
%
%  Initialize coefficients
%
disp(' ')
disp(' Initialize coefficients')
%for j=1:1
    omega=2.*pi*fn(1);
    omegad=omega*sqrt(1.-(damp^2));
%
    out5 = sprintf(' omega=%g   omegad=%g ',omega,omegad);
    disp(out5);    
%    
    cosd=cos(omegad*dt);
    sind=sin(omegad*dt);
%
    out5 = sprintf(' cosd=%g   sind=%g ',cosd,sind);
    disp(out5);    
%
    domegadt=damp*omega*dt;
    a1(1)=2.*exp(-domegadt)*cosd;
    a2(1)=-exp(-2.*domegadt);
%
    out5 = sprintf(' a1=%g  a2=%g ',a1(1),a2(1));
    disp(out5);  
%
    b1(1)=2.*domegadt;
    b2(1)=omega*dt*exp(-domegadt);    
    b2(1)=b2(1)*( (omega/omegad)*(1.-2.*(damp^2))*sind -2.*damp*cosd );
 %
    out5 = sprintf(' b1=%g   b2=%g ',b1(1),b2(1));
    disp(out5);  
%
    c2(1)=-(dt/omegad)*exp(-domegadt)*sind;
 %
    out5 = sprintf(' c2=%g  ',c2(1));
    disp(out5);  
% 
%end
disp(' ');
disp(' Include residual? ');
disp('  1=yes  2=no ')
ire=input(' ');
%
if(ire==1)
%
%   Add trailing zeros for residual response
%
    disp(' ')
    disp(' Add trailing zeros for residual response')
    tmax=(tmx-tmi) + 2./fn(1);
    limit = round( tmax/dt );
    n=limit;
end
%   
yy=zeros(1,n);
for i=1:length(y)
        yy(i)=y(i);        
end        
%
%  SRS engine
%
disp(' ')
disp(' Calculating acceleration');
for j=1:1
%
    forward=[ b1(j),  b2(j),      0 ];
    back   =[     1, -a1(j), -a2(j) ];
%    
    a_resp=filter(forward,back,yy);
%
    x_pos(j)= max(a_resp);
    x_neg(j)= min(a_resp);
end
%
disp(' ')
disp(' Calculating relative displacement');
for j=1:1
%
    rd_forward=[     0,  c2(j),      0 ];
    rd_back   =[     1, -a1(j), -a2(j) ];
%    
    rd_resp=filter(rd_forward,rd_back,yy);
%
    rd_pos(j)= max(rd_resp);
    rd_neg(j)= min(rd_resp);
end
%
disp(' ')
disp(' ')
disp(' Acceleration Response ')
mu=mean(a_resp);
sd=std(a_resp);
arms=sqrt(sd^2+mu^2);
peak=max(abs(a_resp));
if(abs(x_neg(1))>peak)
    peak=abs(x_neg(1));
end
%
clear aar;
aar=a_resp-mu;
aar=aar.^4;
kurtosis=sum(aar)/(n*sd^4);
%
 out9=sprintf('\n absolute peak = %10.2f G \n',peak);
disp(out9);
%
out10=sprintf('\n      maximum = %10.2f G',x_pos(1));
  out11=sprintf('      minimum = %10.2f G',x_neg(1));
  out12=sprintf('      overall = %10.2f GRMS',arms);
out13=sprintf('\n         mean = %10.2f G',mu);
  out14=sprintf('      std dev = %10.2f GRMS',sd);
  out15=sprintf(' crest factor = %10.2f',peak/sd);
  out16=sprintf('     kurtosis = %10.2f',kurtosis);  
%
disp(out10)
disp(out11)
disp(out12)
disp(out13)
disp(out14)
disp(out15)
disp(out16)
%
disp(' ')
disp(' Relative Displacement Response ')
mu=mean(rd_resp);
sd=std(rd_resp);
rd_rms=sqrt(sd^2+mu^2)*386.;
out10=sprintf('\n maximum = %10.3g inch',rd_pos(1)*386.);
out11=sprintf(' minimum = %10.3g inch',rd_neg(1)*386.);
out12=sprintf(' overall = %10.3g inch RMS',rd_rms);
disp(out10)
disp(out11)
disp(out12)
%
nn=length(a_resp);
%
tmin=t(1);
tmax=(t(1)+(nn-1)*dt);
%
tt=linspace(tmin,tmax,nn);
ymin=1.3*min(a_resp);
ymax=1.3*max(a_resp);
if(abs(ymin)>abs(ymax))
    ymax=abs(ymin); 
end
if(abs(ymax)>abs(ymin))
    ymin=-abs(ymax); 
end
%    
%  Output 
%
disp(' ')
disp('Output Acceleration Response Text File?')
choice=input(' 1=yes  2=no  ' );
disp(' ')
%
if choice == 1 
%
      [writefname, writepname] = uiputfile('*.dat','Save acceleration data as');
	  writepfname = fullfile(writepname, writefname);
	  writedata = [tt' a_resp'];
	  fid = fopen(writepfname,'w');
	  fprintf(fid,'  %g     %g\n',writedata');
	  fclose(fid);
%    
%   disp(' Enter output filename ');
% TH_filename = input(' ','s');
%
%  fid = fopen(TH_filename,'w');
%   for i=1:nn
%        fprintf(fid,'%12.5f %10.3f \n',tt(i),a_resp(i));
%   end
%   fclose(fid);
end
%
disp('Plot Acceleration Time History?')
choice=input(' 1=yes  2=no  ' );
disp(' ')
if choice == 1 
    figure(1);
    disp(' Plotting output..... ')
    plot(tt,a_resp);
    axis([tmin,tmax,ymin,ymax]);
    ylabel('Accel (G)');
    xlabel('Time (sec)');
    Q=1./(2.*damp);
    out5 = sprintf(' SDOF Acceleration Response   fn=%g Hz   Q=%g ',fn(1),Q);
    title(out5);
    grid;
    zoom on;
end
%
disp(' ')
disp(' Plot histogram? ')
choice = input(' 1=yes 2=no ');
%
if(choice == 1)
        while(1)
            disp(' ');
            nbar=input(' Enter the number of bars ');
            figure(2);    
            hist(a_resp,nbar)
            title(' Histogram ');
            ylabel(' Counts ');
            disp(' ');
            disp(' Re-plot with different number of bars? ');
            disp(' 1=yes  2=no ');
            irp=input(' ');
            if(irp==2)
                break;
            end
        end
end
%
disp(' ')
disp('Plot Relative Displacement Time History?')
choice=input(' 1=yes  2=no  ' );
disp(' ')
if choice == 1
    figure(3);
    disp(' Plotting output..... ')
    plot(tt,rd_resp*386.);
    ymin=1.3*min(rd_resp);
    ymax=1.3*max(rd_resp);
    ymin=ymin*386.;
    ymax=ymax*386.;
    if(abs(ymin)>abs(ymax))
        ymax=abs(ymin); 
    end
    if(abs(ymax)>abs(ymin))
        ymin=-abs(ymax); 
    end    
    axis([tmin,tmax,ymin,ymax]);    
    ylabel('Relative Disp (inch)');
    xlabel('Time (sec)');
    Q=1./(2.*damp);
    out5 = sprintf(' SDOF Relative Displacement Response   fn=%g Hz   Q=%g ',fn(1),Q);
    title(out5);
    grid;
    zoom on;
end
%
disp(' ');
disp(' plot phase portrait?');
k=input(' 1=yes  2=no ');
if(k==1)
    figure(4);
    plot(rd_resp*386,a_resp);
    grid on;
%   
    ylabel('Acceleration (G)');
    xlabel('Relative Displacement (in)');   
%
end
%
end
%
clear acceleration;
clear rel_disp;
disp(' ');
disp(' Acceleration response matrix:  acceleration');
acceleration(:,1)=tt';
acceleration(:,2)=a_resp';
disp(' ');
disp(' Relative displacement response matrix:  rel_disp');
rel_disp(:,1)=tt';
rel_disp(:,2)=(rd_resp*386.)';